@@ -1,5 +1,5 @@
 /*
- * $Id: fc-case.c,v 1.1.4.1 2005/09/22 23:45:53 plam Exp $
+ * $Id$
  *
  * Copyright © 2004 Keith Packard
  *
@@ -102,10 +102,10 @@ utf8_size (FcChar32 ucs4)
 }
 
 static FcChar8	*foldChars;
-int		nfoldChars;
-int		maxFoldChars;
-FcChar32	minFoldChar;
-FcChar32	maxFoldChar;
+static int	nfoldChars;
+static int	maxFoldChars;
+static FcChar32	minFoldChar;
+static FcChar32	maxFoldChar;
 
 static void
 addChar (FcChar32 c)
